package ru.ip.labs.labs.films.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import ru.ip.labs.labs.films.models.Actor;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.models.Genre;
import ru.ip.labs.labs.films.repository.ActorRepository;
import ru.ip.labs.labs.films.repository.FilmRepository;
import ru.ip.labs.labs.films.repository.GenreRepository;

import javax.persistence.EntityNotFoundException;
import java.util.List;
import java.util.Optional;

@Service
public class FilmsService {
    private FilmRepository repo;
    private GenreService genreService;
    private ActorService actorService;

    public FilmsService(FilmRepository repo, ActorService actorService, GenreService genreService) {
        this.repo = repo;
        this.actorService = actorService;
        this.genreService = genreService;
    }

    @Transactional
    public Film addFilm(String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Student name is null or empty");
        }

        Film film = new Film(name);
        return repo.save(film);
    }

    @Transactional
    public Film addFilm(String name, List<String> genres, List<String> fullNames) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Student name is null or empty");
        }

        Film film = new Film(name);
        Film result = repo.save(film);

        updateGenres(result.getId(), genres);
        return updateActors(film.getId(), fullNames);
    }

    // фильмы по жанру
    // фильмы по актеру

    @Transactional
    public Film addGenre(Long filmId, Long genreId) {
        final Optional<Film> filmOpt = repo.findById(filmId);

        if (filmOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("Film with id [%s] is not found", filmId));
        }

        Film film = filmOpt.get();

        final Genre genre = genreService.findGenre(genreId);
        if (genre == null) {
            throw new EntityNotFoundException(String.format("Genre with id [%s] is not found", genreId));
        }

        film.addGenre(genre);
        return repo.save(film);
    }

    @Transactional
    public Film updateGenres(Long filmId, List<String> genres) {
        final Optional<Film> filmOpt = repo.findById(filmId);

        if (filmOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("Film with id [%s] is not found", filmId));
        }

        Film film = filmOpt.get();

        film.deleteGenres();

        for(String g : genres) {
            Genre genre = genreService.findByName(g);
            film.addGenre(genre);
        }

        return repo.save(film);
    }

    @Transactional
    public Film updateActors(Long filmId, List<String> fullNames) {
        final Optional<Film> filmOpt = repo.findById(filmId);

        if (filmOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("Film with id [%s] is not found", filmId));
        }

        Film film = filmOpt.get();

        film.deleteActors();

        for(String fn : fullNames) {
            String arr[] = fn.split(" ");
            Actor actor = actorService.findByFullName(arr[0], arr[1]);
            film.addActor(actor);
        }

        return repo.save(film);
    }

    @Transactional
    public Film addActor(Long filmId, Long actorId) {
        final Optional<Film> filmOpt = repo.findById(filmId);

        if (filmOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("Film with id [%s] is not found", filmId));
        }

        Film film = filmOpt.get();

        final Actor actor = actorService.findActor(actorId);
        if (actor == null) {
            throw new EntityNotFoundException(String.format("Genre with id [%s] is not found", actorId));
        }

        film.addActor(actor);
        return repo.save(film);
    }

    @Transactional(readOnly = true)
    public Film findFilm(Long id) {
        final Optional<Film> film = repo.findById(id);
        if (film.isEmpty()) {
            throw new EntityNotFoundException(String.format("Film with id [%s] is not found", id));
        }
        return film.get();
    }

    @Transactional(readOnly = true)
    public List<Film> findAllFilms() {
        return repo.findAll();
    }

    @Transactional
    public Film updateFilm(Long id, String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Film name is null or empty");
        }
        final Optional<Film> currentFilmOpt = repo.findById(id);

        if(currentFilmOpt.isEmpty()) {
            return null;
        }

        final Film currentFilm = currentFilmOpt.get();

        currentFilm.setName(name);
        return repo.save(currentFilm);
    }

    @Transactional
    public Film updateFilm(Long id, String name, List<String> genres, List<String> fullNames) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Film name is null or empty");
        }
        final Optional<Film> currentFilmOpt = repo.findById(id);

        if(currentFilmOpt.isEmpty()) {
            return null;
        }

        final Film currentFilm = currentFilmOpt.get();

        currentFilm.setName(name);
        repo.save(currentFilm);

        updateGenres(id, genres);
        return updateActors(id, fullNames);
    }

    @Transactional
    public Film deleteFilm(Long id) {
        final Optional<Film> currentFilm = repo.findById(id);
        if(currentFilm.isEmpty()) {
            return null;
        }

        repo.deleteById(id);
        return currentFilm.get();
    }

    @Transactional
    public void deleteAllFilms() {
        repo.deleteAll();
    }
}
